
import pandas as pd
from voyant_indicators.volatility.atr import atr


def natr(df, high, low, close, natr, n):
    """
    The Normalized Average True Range (NATR) is a measure of volatility that is
    directly comparable across assets with different price levels.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        atr (string): the column name for the atr values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with average true range of the asset calculated.

    """

    df = atr(df, high, low, close, "average_true_range", n)
    df[natr] = 100 * (df["average_true_range"] / df[close])

    return df




import pandas as pd


from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def t3(df, price, t3, n, v_factor=0.7):
    """
    The T3 is a type of moving average, or smoothing function. It is based on the
    DEMA. The T3 takes the DEMA calculation and adds a vfactor which is between
    zero and 1. The resultant function is called the GD, or Generalized DEMA. A
    GD with vfactorof 1 is the same as the DEMA. A GD with a vfactor of zero is
    the same as an Exponential Moving Average. The T3 typically uses a vfactor
    of 0.7.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        t3 (string): the column name for the t3 moving average results.
        n (int): the total number of periods.
        v_factor (float): v factor is a volume factor between 0 and 1 which
                          determines how the moving averages responds.

    Returns:
        df (pd.DataFrame): Dataframe with t3 moving average of the asset calculated.

    """

    def gd(df, price, gd, n):
        df = ema(df, price, gd + "_ema", n)
        df = ema(df[n - 1 :], gd + "_ema", gd + "_ema_2", n)
        df[gd] = (1 + v_factor) * df[gd + "_ema"] - v_factor * df[
            gd + "_ema_2"
        ]
        df = df.dropna().reset_index(drop=True)
        df.drop([gd + "_ema", gd + "_ema_2"], axis=1, inplace=True)
        return df

    df = gd(df, price, t3 + "_gd_1", n)
    df = gd(df, t3 + "_gd_1", t3 + "_gd_2", n)
    df = gd(df, t3 + "_gd_2", t3, n)
    df.drop([t3 + "_gd_1", t3 + "_gd_2"], axis=1, inplace=True)

    return df



def parabolic_sar(df, high,low,close, col, initial_af=0.02,step_af=0.02, end_af=0.2):
        
  
    df['trend']=0
    df['sar']=0.0
    df['ep']=0.0
    df['af']=0.0
    df[col]=0.0

    #initial values for recursive calculation
    df['trend'][1]=1 if df[close][1]>df[close][0] else -1
    df['sar'][1]=df[high][0] if df['trend'][1]>0 else df[low][0]
    df.at[1,'real sar']=df['sar'][1]
    df['ep'][1]=df[high][1] if df['trend'][1]>0 else df[low][1]
    df['af'][1]=initial_af

    #calculation
    for i in range(2,len(df)):
        
        temp=df['sar'][i-1]+df['af'][i-1]*(df['ep'][i-1]-df['sar'][i-1])
        if df['trend'][i-1]<0:
            df.at[i,'sar']=max(temp,df[high][i-1],df[high][i-2])
            temp=1 if df['sar'][i]<df[high][i] else df['trend'][i-1]-1
        else:
            df.at[i,'sar']=min(temp,df[low][i-1],df[low][i-2])
            temp=-1 if df['sar'][i]>df[low][i] else df['trend'][i-1]+1
        df.at[i,'trend']=temp
    
        
        if df['trend'][i]<0:
            temp=min(df[low][i],df['ep'][i-1]) if df['trend'][i]!=-1 else df[low][i]
        else:
            temp=max(df[high][i],df['ep'][i-1]) if df['trend'][i]!=1 else df[high][i]
        df.at[i,'ep']=temp
    
    
        if np.abs(df['trend'][i])==1:
            temp=df['ep'][i-1]
            df.at[i,'af']=initial_af
        else:
            temp=df['sar'][i]
            if df['ep'][i]==df['ep'][i-1]:
                df.at[i,'af']=df['af'][i-1]
            else:
                df.at[i,'af']=min(end_af,df['af'][i-1]+step_af)
        df.at[i,'real sar']=temp
       
        
    return df


import pandas as pd
import numpy as np


def ema(df, price, col, n):
    """
    Exponential Moving Average (EMA) is a Weighted Moving Average (WMA) that
    gives more weighting to recent price data than Simple Moving Average (SMA)
    does.

    The EMA formula is based on the previous day EMA value. Since we have to
    start our calculation somewhere, the initial value for our first EMA will
    actually be an SMA.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        col (string): the column name for the n-day exponential moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with n-day exponential moving average of the asset calculated.

    Example Strategy:
    Bullish

    AND [Daily SMA(20,Daily Volume) > 40000] 
    AND [Daily SMA(60,Daily Close) > 20] 

    AND [Daily SMA(150,Daily Close) > 5 days ago Daily SMA(150,Daily Close)] 
    AND [Daily EMA(5,Daily Close) > Daily EMA(35,Daily Close)] 
    AND [Yesterday's Daily EMA(5,Daily Close) < Yesterday's Daily EMA(35,Daily Close)] 
    AND [Daily Volume > Daily SMA(200,Daily Volume)]

    Bearish:

    AND [Daily SMA(20,Daily Volume) > 40000] 
    AND [Daily SMA(60,Daily Close) > 20] 

    AND [Daily SMA(150,Daily Close) < 5 days ago Daily SMA(150,Daily Close)] 
    AND [Daily EMA(5,Daily Close) < Daily EMA(35,Daily Close)] 
    AND [Yesterday's Daily EMA(5,Daily Close) > Yesterday's Daily EMA(35,Daily Close)] 
    AND [Daily Volume > Daily SMA(200,Daily Volume)]

    Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:moving_averages
    
    """

    df = df.copy().reset_index(drop=True)

    if df.shape[0] < n:
        df[col] = None
        return df

    k = 2.0 / (n + 1)
    prev_ema = list(df[:n][price].rolling(window=n).mean())[-1]
    df.loc[n - 1, col] = prev_ema
    df.loc[n:, col] = 0.0
    emas = [0.0 for i in range(n)]
    for row in df.loc[n:, [price]].itertuples(index=False):
        emas.append((k * row[0]) + ((1 - k) * prev_ema))
        prev_ema = emas[-1]
    df[col] += emas

    return df




import pandas as pd


def sma(df, price, col, n):
    """
    Simple Moving Average (SMA) is an arithmetic moving average calculated by
    adding recent closing prices then dividing that by the number of time periods
    in the calculation average.

    SMA = (P_1 + ... + P_n) / n

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        col (string): the column name for the n-day moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with n-day moving average of the asset calculated.

    Example Strategy:
    Bullish

    AND [Daily SMA(20,Daily Volume) > 40000] 
    AND [Daily SMA(60,Daily Close) > 20] 

    AND [Daily SMA(150,Daily Close) > 5 days ago Daily SMA(150,Daily Close)] 
    AND [Daily EMA(5,Daily Close) > Daily EMA(35,Daily Close)] 
    AND [Yesterday's Daily EMA(5,Daily Close) < Yesterday's Daily EMA(35,Daily Close)] 
    AND [Daily Volume > Daily SMA(200,Daily Volume)]

    Bearish:

    AND [Daily SMA(20,Daily Volume) > 40000] 
    AND [Daily SMA(60,Daily Close) > 20] 

    AND [Daily SMA(150,Daily Close) < 5 days ago Daily SMA(150,Daily Close)] 
    AND [Daily EMA(5,Daily Close) < Daily EMA(35,Daily Close)] 
    AND [Yesterday's Daily EMA(5,Daily Close) > Yesterday's Daily EMA(35,Daily Close)] 
    AND [Daily Volume > Daily SMA(200,Daily Volume)]

    Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:moving_averages

    """
    df = df.copy()

    if df.shape[0] < n:
        df[col] = None
    
    else:
        df[col] = df[price].rolling(window=n).mean()

    return df





import pandas as pd
from voyant_indicators.volume.ad import ad
from voyant_indicators.trend.ema import ema


def adosc(df, high, low, close, volume, col, fast_period, slow_period):
    """
    Developed by Marc Chaikin, the Chaikin Oscillator measures the momentum of the Accumulation Distribution Line using the MACD formula. 
    (This makes it an indicator of an indicator.) The Chaikin Oscillator is the difference between the 3-day and 10-day EMAs 
    of the Accumulation Distribution Line. Like other momentum indicators, this indicator is designed to anticipate directional changes 
    in the Accumulation Distribution Line by measuring the momentum behind the movements. A momentum change is the first step to a trend change.
    Anticipating trend changes in the Accumulation Distribution Line can help chartists anticipate trend changes in the underlying security. 
    The Chaikin Oscillator generates signals with crosses above/below the zero line or with bullish/bearish divergences.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        volume (string): the column name for the volume of the asset.
        adosc (string): the column name for the adosc values.
        fast_period (int): the time period of the fast exponential moving average.
        slow_period (int): the time period of the slow exponential moving average.

    Returns:
        df (pd.DataFrame): Dataframe with adosc of the asset calculated.

    Sample Strategy:
    
    Bullish:

        AND [Daily SMA(60,Daily Volume) > 100000] 
        AND [Daily SMA(60,Daily Close) > 10] 

        AND [Daily Chaikin Osc(3,10) crosses 1000] 
        AND [Daily RSI(14,Daily Close) crosses 55]

    Bearish:

        AND [Daily SMA(60,Daily Volume) > 100000] 
        AND [Daily SMA(60,Daily Close) > 10] 

        AND [-1000 crosses Daily Chaikin Osc(3,10)] 
        AND [45 crosses Daily RSI(14,Daily Close)]

    Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:chaikin_oscillator


    """

    df = ad(df, high, low, close, volume, col + "_ad")
    df = ema(df, col + "_ad", col + "_ad_fast", fast_period)
    df = ema(df, col + "_ad", col + "_ad_slow", slow_period)
    df[col] = df[col + "_ad_fast"] - df[col + "_ad_slow"]

    if df.shape[0] > slow_period:
        df = df.dropna().reset_index(drop=True)

    df.drop(
        [col + "_ad", col + "_ad_fast", col + "_ad_slow"],
        axis=1,
        inplace=True,
    )

    return df



import pandas as pd
def donchian_channel(df, high, low, col, n):
    """Calculate donchian channel of given pandas data frame.
    :param df: pandas.DataFrame
    :param n:
    :return: pandas.DataFrame
    """
    i = 0
    dc_l = []
    while i < n - 1:
        dc_l.append(0)
        i += 1

    i = 0
    while i + n - 1 < df.index[-1]:
        dc = max(df[high].ix[i:i + n - 1]) - min(df[low].ix[i:i + n - 1])
        dc_l.append(dc)
        i += 1

    donchian_chan = pd.Series(dc_l, name=col)
    donchian_chan = donchian_chan.shift(n - 1)
    return df.join(donchian_chan)



import numpy as np
import pandas as pd


def aroon(df, high, low, aroon, n):
    """
    Aroon is an indicator system that determines whether a stock is trending or
    not and how strong the trend is. The Aroon indicators measure the number of
    periods since price recorded an x-day high or low. There are two separate
    indicators: Aroon-Up and Aroon-Down. Aroon indicators are quite different
    from typical momentum oscillators, which focus on price relative to time.
    Aroon is unique because it focuses on time relative to price. Aroon indicators
    can be use to spot emerging trends, identify consolidations, define correction
    periods and anticipate reversals.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        aroon (string): the column name for the results of aroon up and aroon down.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with aroon the asset calculated.

    """

    df[aroon + "_up"] = (
        df[high]
        .rolling(n)
        .apply(lambda x: float(np.argmax(x) + 1) / n * 100, raw=True)
    )
    df[aroon + "_dn"] = (
        df[low]
        .rolling(n)
        .apply(lambda x: float(np.argmin(x) + 1) / n * 100, raw=True)
    )
    df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd


def mom(df, price, col, n):
    """
    The Momentum indicator is a speed of movement indicator designed to identify the speed (or strength) of price movement. 
    The momentum indicator compares the most recent closing price to a previous closing price (can be the closing price of any time frame). 
    Common ways to use Momentum Indicator:
    Buy strategy: 14 MOM crosses above 0
    Sell strategy: 14 MOM crosses below 0.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name of the price of the asset.
        col (string): the column name for the rate of change values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with mom of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    df[col] = df[price] - df[price].shift(n)
    #df = df.dropna().reset_index(drop=True)

    return df




import pandas as pd
from voyant_indicators.trend.ema import ema
from voyant_indicators.util.trange import trange


def atr(df, high, low, close, atr, n):
    """
    The Average True Range (ATR) is a Welles Wilder style moving average of the
    True Range. The ATR is a measure of volatility. High ATR values indicate
    high volatility, and low values indicate low volatility, often seen when
    the price is flat.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        atr (string): the column name for the atr values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with average true range of the asset calculated.

    """

    df = trange(df, high, low, close, "true_range")
    prev_atr = df.loc[0 : n - 1, "true_range"].sum() / n
    df.loc[n - 1, atr] = prev_atr
    df = df.drop(df.index[: n - 1]).reset_index(drop=True)
    df = df.fillna(0)
    avg_trs = [0.0]
    for row in df.loc[1:, ["true_range", atr]].itertuples(index=False):
        avg_trs.append((prev_atr * (n - 1) + row[0]) / n)
        prev_atr = avg_trs[-1]
    df[atr] += avg_trs

    return df



import pandas as pd


def bop(df, open, high, low, close, col):
    """
    Balance of Power (BOP) is used in technical analysis to evaluate the
    strength of buyers and sellers by assessing the ability to push price to
    extreme high and low levels. The Balance of Power oscillates around 0 (zero)
    center line in the range from -1 to +1. Positive BOP readings are considered
    as a bullish sign and negative BOP readings is a Bearish sign.
    
    Balance of Power = (Close price – Open price) / (High price – Low price)


    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        open (string): the column name for the opening price of the asset.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        col (string): the column name for the bop values.

    Returns:
        df (pd.DataFrame): Dataframe with balance of power calculated.

    """

    df = df.copy()


    df[col] = (df[close] - df[open]) / (df[high] - df[low])

    return df



import pandas as pd


from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema
from voyant_indicators.util.trange import trange


def ultosc(df,high,low, close, col, time_period_1=7, time_period_2=14,time_period_3=28):
    """
    The Ultimate Oscillator (ULTOSC) by Larry Williams is a momentum oscillator
    that incorporates three different time periods to improve the overbought and
    oversold signals.
    
    There are three steps to calculating the Ultimate Oscillator. This example uses 7, 14, 28 parameters:

    1. Before calculating the Ultimate Oscillator, two variable need to be defined; Buying Pressure and True Range.

    Buying Pressure (BP) = Close - Minimum (Lowest between Current Low or Previous Close)
    True Range (TR) = Maximum (Highest between Current High or Previous Close) - Minimum (Lowest between Current Low or Previous Close)

    2. The Ultimate Oscillator then uses these figures over three time periods:

    Average7 = (7 Period BP Sum) / (7 Period TR Sum)
    Average14 = (14 Period BP Sum) / (14 Period TR Sum)
    Average28 = (28 Period BP Sum) / (28 Period TR Sum)

    3. The final Ultimate Oscillator calculations can now be made:

    UO = 100 x [(4 x Average7)+(2 x Average14)+Average28]/(4+2+1)

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        ultosc (string): the column name for the ultimate oscillator values.
        time_period_1 (int): The first time period for the indicator. By default, 7.
        time_period_2 (int): The second time period for the indicator. By default, 14.
        time_period_3 (int): The third time period for the indicator. By default, 28.

    Returns:
        df (pd.DataFrame): Dataframe with ultimate oscillator of the asset calculated.

    Strategy:
        Bullish UO Divergence

        Bullish Divergence forms meaning price forms a lower low while UO makes a higher low.
        The low of the Divergence should be below 30.
        UO breaks above the high of the Divergence.


        Bearish UO Divergence

        Bearish Divergence forms meaning price forms a higher high while UO makes a lower high.
        The high of the Divergence should be above 70.
        UO falls below the low of the Divergence.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < time_period_3:
        df[col] = None
        return df


    df[col + "previous_close"] = df[close].shift(1)
    df = trange(df, high, low, close, col + "_true_range")
    df = df.dropna().reset_index(drop=True)
    df[col + "_true_low"] = df[[low, col + "previous_close"]].min(axis=1)
    df[col + "_close-tl"] = df[close] - df[col + "_true_low"]
    df = sma(df, col + "_close-tl", col + "_a1", time_period_1)
    df = sma(df, col + "_true_range", col + "_b1", time_period_1)
    df = sma(df, col + "_close-tl", col + "_a2", time_period_2)
    df = sma(df, col + "_true_range", col + "_b2", time_period_2)
    df = sma(df, col + "_close-tl", col + "_a3", time_period_3)
    df = sma(df, col + "_true_range", col + "_b3", time_period_3)
    a1_b1 = df[col + "_a1"] / df[col + "_b1"]
    a2_b2 = df[col + "_a2"] / df[col + "_b2"]
    a3_b3 = df[col + "_a3"] / df[col + "_b3"]
    df[col] = 100 * ((4 * a1_b1) + (2 * a2_b2) + a3_b3) / 7.0
    df.drop(
        [
            col + "_true_range",
            col + "previous_close",
            col + "_true_low",
            col + "_close-tl",
            col + "_a1",
            col + "_b1",
            col + "_a2",
            col + "_b2",
            col + "_a3",
            col + "_b3",
        ],
        axis=1,
        inplace=True,
    )
    #df = df.dropna().reset_index(drop=True)
    return df




import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def trima(df, price, trima, n):
    """
    The Triangular Moving Average (TRIMA) is similar to other moving averages in
    that it shows the average (or mean) price over a specified number of data
    points (usually a number of price bars). However, the triangular moving average
    differs in that it is double smoothed—which also means averaged twice.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        trima (string): the column name for the n-day double exponential moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with triangular moving average of the asset calculated.

    """

    first_period_sma = None
    second_period_sma = None
    if n % 2 == 0:
        first_period_sma = int((n / 2) + 1)
        second_period_sma = int(n / 2)
    else:
        first_period_sma = int((n + 1) / 2)
        second_period_sma = int((n + 1) / 2)
    df = sma(df, price, trima + "_sma", first_period_sma)
    df = sma(
        df[first_period_sma - 1 :], trima + "_sma", trima, second_period_sma
    )
    df = df.dropna().reset_index(drop=True)
    df.drop([trima + "_sma"], axis=1, inplace=True)

    return df



import pandas as pd
from indicators.momentum.rsi import rsi

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema

def stoch(df, high ,low, price,col,n,ma_type,ma_n):
    """
    The Stochastic Oscillator is another well-known momentum indicator used in
    technical analysis. The idea behind this indicator is that the closing
    prices should predominantly close in the same direction as the prevailing
    trend.

    In an upward trend the price should be closing near the highs of the trading
    range and in a downward trend the price should be closing near the lows of
    the trading range. When this occurs it signals continued momentum and
    strength in the direction of the prevailing trend.

    StochRSI = (RSI - Lowest Low RSI) / (Highest High RSI - Lowest Low RSI)

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the price  of the asset.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period low price  of the asset.
        col (string): the column name for the result
        n: Time period to consider

    Returns:
        df (pd.DataFrame): Dataframe with slow %k and slow %d of the asset calculated.

    Strategy:
        Bullish:
        - AND [Daily SMA(20,Daily Volume) > 40000] 
          AND [Daily SMA(60,Daily Close) > 10] 

          AND [Daily SMA(10,Daily Close) > Daily SMA(60,Daily Close)] 
          AND [Daily Stoch RSI(14) < 0.1] 
          AND [Daily Close < Daily SMA(10,Daily Close)]
        
        Bearish:
        - AND [Daily SMA(20,Daily Volume) > 40000] 
          AND [Daily SMA(60,Daily Close) > 10] 

          AND [Daily SMA(10,Daily Close) < Daily SMA(60,Daily Close)] 
          AND [Daily Stoch RSI(14) > 0.9] 
          AND [Daily Close > Daily SMA(10,Daily Close)]

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    df = rsi(df,price,"rsi",n)
    hh = df[high].rolling(window=n).max()
    ll = df[low].rolling(window= n).min()
    df[col] = (df["rsi"] - ll) / (hh - ll)

    if ma_type == -1:
        return df
    else:
        ma_types = {0: sma, 1: ema, 2: wma, 3: dema, 4: tema}
        df.rename(columns = {col:col+"_prev"}, inplace = True) 

        df = ma_types[ma_type]( df, col+"_prev", col, ma_n)
    
    
    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd


def rsi(df, price, col, n):
    """
    The Relative Strength Index (RSI) is a momentum indicator that measures the
    magnitude of recent price changes to evaluate overbought or oversold
    conditions in the price of a stock or other asset.

    First calculations for average gain and average loss are simple n-period averages,
        1st average gain = sum of gains over the n periods / n
        1st average loss = sum of losses over the n periods / n

    Subsequent calculations are based on prior averages and the current gains / losses,
        average gain = (previous gain * (n - 1) + current gain) / n
        average loss = (previous loss * (n - 1) + current loss) / n

    RSI = 100 * (average gain / (average gain + average loss))

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        col (string): the column name for the rsi values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with rsi of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    df["diff"] = df[price].diff()
    df.loc[1:, "up"] = df[1:].loc[df["diff"] > 0, "diff"]
    df.loc[1:, "dn"] = -df[1:].loc[df["diff"] < 0, "diff"]
    prev_avg_gain = df[: n + 1].loc[df["diff"] > 0, "diff"].sum() / n
    prev_avg_loss = -df[: n + 1].loc[df["diff"] < 0, "diff"].sum() / n
    df.loc[n, "avg_gain"] = prev_avg_gain
    df.loc[n, "avg_loss"] = prev_avg_loss
    #df = df.drop(df.index[:n]).reset_index(drop=True)
    df = df.fillna(0)
    avg_gains, avg_losses = [0.0], [0.0]
    for row in df.loc[1:, ["up", "dn"]].itertuples(index=False):
        avg_gains.append((prev_avg_gain * (n - 1) + row[0]) / n)
        avg_losses.append((prev_avg_loss * (n - 1) + row[1]) / n)
        prev_avg_gain, prev_avg_loss = avg_gains[-1], avg_losses[-1]
    df["avg_gain"] += avg_gains
    df["avg_loss"] += avg_losses
    df[col] = 100 * df["avg_gain"] / (df["avg_gain"] + df["avg_loss"])
    df.drop(["diff", "up", "dn", "avg_gain", "avg_loss"], axis=1, inplace=True)

    #print(df)
    return df




import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def macdext(
    df,
    price,
    macdext,
    fast_period,
    slow_period,
    signal_period,
    fast_ma_type=0,
    slow_ma_type=0,
    signal_ma_type=0,
):
    """
    The Moving Average Convergence Divergence (MACD) is the difference between
    two Exponential Moving Averages. The Signal line is an Exponential Moving
    Average of the MACD. The MACD signals trend changes and indicates the start
    of new trend direction. High values indicate overbought conditions, low values
    indicate oversold conditions. Divergence with the price indicates an end to
    the current trend, especially if the MACD is at extreme high or low values.
    When the MACD line crosses above the signal line a buy signal is generated.
    When the MACD crosses below the signal line a sell signal is generated. To
    confirm the signal, the MACD should be above zero for a buy, and below zero
    for a sell.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the series type of the asset.
        macd (string): the column name for the macd results.
        close (string): the column name for the closing price of the asset.
        fast_period (int): the time period of the fast exponential moving average.
        slow_period (int): the time period of the slow exponential moving average.
        signal_period (int): the time period of the macd signal.
        fast_ma_type (int): moving average type for the fast moving average.
        slow_ma_type (int): moving average type for the slow moving average.
        signal_ma_type (int): moving average type for the signal moving average.

    Returns:
        df (pd.DataFrame): Dataframe with macd of the asset calculated.

    """

    ma_types = {0: sma, 1: ema, 2: wma, 3: dema, 4: tema}

    df = ma_types[fast_ma_type](df, price, macdext + "_fast_ema", fast_period)
    df = ma_types[slow_ma_type](df, price, macdext + "_slow_ema", slow_period)
    df[macdext] = df[macdext + "_fast_ema"] - df[macdext + "_slow_ema"]
    df = ma_types[signal_ma_type](
        df[slow_period - 1 :], macdext, macdext + "_signal", signal_period
    )
    df[macdext + "_hist"] = df[macdext] - df[macdext + "_signal"]
    df.drop(
        [macdext + "_fast_ema", macdext + "_slow_ema"], axis=1, inplace=True
    )
    df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd
from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def ppo(df, price, col, fast_period, slow_period, ma_type):
    """
    The Percentage Price Oscillator (PPO) is a momentum oscillator that measures
    the difference between two moving averages as a percentage of the larger
    moving average.
    
        PPO = (MA_long - MA_Slow)/MA_Slow

    One advantage to the percentage price oscillator is that it's a dimensionless quantity, 
    a pure number that isn't fixed to a value such as the price of the underlying stock or other security. 
    Also, because the percentage price oscillator compares two exponential moving averages, 
    it lets the user compare movements through different time frames. 
    The price of the security itself becomes almost of secondary importance.
    
    For analysts who choose to use the percentage price oscillator, a value outside the range of -10% to +10% 
    is supposed to indicate a stock being oversold or overbought, respectively.


    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name of the price of the asset.
        col (string): the column name for the % price oscillator values.
        fast_period (int): the time period of the fast moving average.
        slow_period (int): the time period of the slow moving average.
        ma_type (int): Moving average type.

    Returns:
        df (pd.DataFrame): Dataframe with ppo of the asset calculated.

    """
    df = df.copy().reset_index(drop=True)
    if df.shape[0] < slow_period:
        df[col] = None
        return df

    ma_types = {"sma":sma, "ema" : ema, "wma": wma, "dema": dema, "tem": tema}

    df = ma_types[ma_type](df, price, col + "_fast", fast_period)
    df = ma_types[ma_type](df, price, col + "_slow", slow_period)
    df[col] = ((df[col + "_fast"] - df[col + "_slow"]) / df[col + "_slow"]) * 100
    df = df.dropna().reset_index(drop=True)

    return df




import pandas as pd


def obv(df, price, volume, col):
    """
    On Balance Volume (OBV) measures buying and selling pressure as a cumulative indicator, adding volume on up days and subtracting it on down days
    The On Balance Volume (OBV) line is simply a running total of positive and negative volume. 
    A period's volume is positive when the close is above the prior close and is negative when the close is below the prior close.
    
    The On Balance Volume (OBV) is a cumulative total of the up and down volume.
    When the close is higher than the previous close, the volume is added to
    the running total, and when the close is lower than the previous close,
    the volume is subtracted from the running total.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        volume (string): the column name of the volume of the asset.
        obv (string): the column name for the on balance volume values.

    Returns:
        df (pd.DataFrame): Dataframe with obv of the asset calculated.

    Sample Strategy:
    Bullish:
        AND [Daily SMA(60,Daily Volume) > 100000] 
        AND [Daily SMA(60,Daily Close) > 10] 

        AND [Daily Close < Daily SMA(65,Daily Close)] 
        AND [Daily AccDist > Daily AccDist Signal (65)] 
        AND [Daily OBV > Daily OBV Signal(65)] 
        AND [Daily Close < Daily SMA(20,Daily Close)] 
        AND [Daily AccDist > Daily AccDist Signal (20)] 
        AND [Daily OBV > Daily OBV Signal(20)]

    Bearish:

        AND [Daily SMA(60,Daily Volume) > 100000] 
        AND [Daily SMA(60,Daily Close) > 10] 

        AND [Daily Close > Daily SMA(65,Daily Close)] 
        AND [Daily AccDist < Daily AccDist Signal (65)] 
        AND [Daily OBV < Daily OBV Signal(65)] 
        AND [Daily Close > Daily SMA(20,Daily Close)] 
        AND [Daily AccDist < Daily AccDist Signal (20)] 
        AND [Daily OBV < Daily OBV Signal(20)]
    
    Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:on_balance_volume_obv


    """

    df["diff"] = df[price].diff()
    df = df.fillna(1)
    df.loc[df["diff"] > 0, col + "_sign"] = 1
    df.loc[df["diff"] < 0, col + "_sign"] = -1
    df.loc[df["diff"] == 0, col + "_sign"] = 0
    volume_sign = df[volume] * df[col + "_sign"]
    df[col] = volume_sign.cumsum()
    df.drop(["diff", col + "_sign"], axis=1, inplace=True)

    return df



import pandas as pd
from voyant_indicators.trend.aroon import aroon


def aroonosc(df, high, low, aroonosc, n):
    """
    An Aroon Oscillator is a trend-following indicator that uses aspects of the
    Aroon Indicator ("Aroon Up" and "Aroon Down") to gauge the strength of a
    current trend and the likelihood that it will continue. The Aroon Oscillator
    is calculated by subtracting Aroon Up from Aroon Down.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        aroonosc (string): the column name for the results of aroonosc.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with aroonosc the asset calculated.

    """

    df = aroon(df, high, low, "aroon", n)
    df[aroonosc] = df["aroon_up"] - df["aroon_dn"]
    df.drop(["aroon_up", "aroon_dn"], axis=1, inplace=True)

    return df




import pandas as pd


def wma(df, price, col, n):
    """
    The Weighted Moving Average calculates a weight for each value in the series.
    The more recent values are assigned greater weights. The Weighted Moving
    Average is similar to a Simple Moving average in that it is not cumulative,
    that is, it only includes values in the time period (unlike an Exponential
    Moving Average). The Weighted Moving Average is similar to an Exponential
    Moving Average in that more recent data has a greater contribution to the
    average.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        col (string): the column name for the n-day weighted moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with n-day weighted moving average of the asset calculated.

    """

    def wa(x):
        return sum([(i + 1) * p for i, p in enumerate(x)]) / (n * (n + 1) / 2)

    df[col] = df[price].rolling(window=n).apply(lambda x: wa(x), raw=True)

    return df



import numpy as np
import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def cci(df, high, low, close, col, n, c=0.015):
    """
    The CCI is designed to detect beginning and ending market trends. The range
    of 100 to -100 is the normal trading range. CCI values outside of this range
    indicate overbought or oversold conditions. You can also look for price divergence
    in the CCI. If the price is making new highs, and the CCI is not, then a price
    correction is likely.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        col (string): the column name for the cci values.
        n (int): the total number of periods.
        c (float): scaling factor to provide more readable numbers, usually 0.015.

    Returns:
        df (pd.DataFrame): Dataframe with commodity channel index calculated.

    """


    df = df.copy()

    if df.shape[0] < n:
        df[col] = None


    df[col + "_tp"] = (df[high] + df[low] + df[close]) / 3.0
    df = sma(df, col + "_tp", col + "_atp", n)
    mdev = (
        df[col + "_tp"]
        .rolling(n)
        .apply(lambda x: np.fabs(x - x.mean()).mean(), raw=True)
    )
    df[col] = (df[col + "_tp"] - df[col + "_atp"]) / (c * mdev)
    df.drop([col + "_tp", col + "_atp"], axis=1, inplace=True)
    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd


def willr(df, high, low, close, col, n):
    """
    The Williams %R is a momentum indicator, which gauges if a stock is overbought or oversold.

    willr = ((Highesst High - Close) / (Highest High - lowest Low)) * -100

    Williams %R is a momentum indicator that reflects the level of the
    close relative to the highest high for the look-back period. %R corrects for the
    inversion by multiplying the raw value by -100. As a result, the Fast Stochastic
    Oscillator and Williams %R produce the exact same lines, only the scaling is different.
    Williams %R oscillates from 0 to -100. Readings from 0 to -20 are considered overbought.
    Readings from -80 to -100 are considered oversold. Unsurprisingly, signals derived
    from the Stochastic Oscillator are also applicable to Williams %R.
    
    Strategy:
    The oscillator has a range of -100 to 0. Readings below -80 represent oversold territory and readings above -20 represent overbought.

    Now, this does not mean you should buy readings below -80 and sell readings above -20.

    During a strong uptrend, a stock can hover around -20. Conversely, in a strong downtrend, a stock can stay in the -80 territory.


    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        col (string): the column name for the willr values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with willr of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    hh = df[high].rolling(window=n).max()
    ll = df[low].rolling(window=n).min()
    df[col] = -100 * (hh - df[close]) / (hh - ll)
    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd


def rocr(df, price, col, n):
    """
    The Rate-of-Change Ratio (ROCR) indicator, which is also referred to as simply
    Momentum, is a pure momentum oscillator that measures the percent change in
    price from one period to the next.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name of the price of the asset.
        rocr (string): the column name for the rate of change ratio.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with rocr of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    df[col] = df[price] / df[price].shift(n)
    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def ao(df, high, low, col, ma_type, fast_period=5, slow_period=34):
    """Awesome Oscillator
    From: https://www.tradingview.com/wiki/Awesome_Oscillator_(AO)
    The Awesome Oscillator is an indicator used to measure market momentum. AO calculates the difference of a 
    34 Period and 5 Period Simple Moving Averages. The Simple Moving Averages that are used are not calculated 
    using closing price but rather each bar's midpoints. AO is generally used to affirm trends or to anticipate 
    possible reversals. 
    
    From: https://www.ifcm.co.uk/ntx-indicators/awesome-oscillator
    Awesome Oscillator is a 34-period simple moving average, plotted through the central points of the bars (H+L)/2, 
    and subtracted from the 5-period simple moving average, graphed across the central points of the bars (H+L)/2.
    MEDIAN PRICE = (HIGH+LOW)/2
    AO = SMA(MEDIAN PRICE, 5)-SMA(MEDIAN PRICE, 34)
    where
    SMA — Simple Moving Average.
    
    Args:
        high(pandas.Series): dataset 'High' column.
        low(pandas.Series): dataset 'Low' column.
        s(int): short period
        l(int): long period
        
    Returns:
        pandas.Series: New feature generated.
    """

    df = df.copy()

    df["mp"] = 0.5 * (df[high] + df[low])

    ma_types = {"sma":sma, "ema" : ema, "wma": wma, "dema": dema, "tem": tema}


    df = ma_types[ma_type](df, "mp", col + "_fast", fast_period)
    df = ma_types[ma_type](df, "mp", col + "_slow", slow_period)


    ao = df[col+"_fast"] - df[col+"_slow"]
    df[col] = ao
    

    return df




import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema

def tema(df, price, tema, n):
    """
    Triple Exponential Moving Average (TEMA) was designed to smooth price
    fluctuations and filter out volatility, thereby making it easier to
    dentify trends without the lag associated with moving averages.

    The TEMA equation is a composite of a single exponential moving average,
    a double exponential moving average, and a triple exponential moving average.

    TEMA = 3 * EMA(p, n) - 3 * EMA(EMA(p, n), n) + EMA(EMA(EMA(p, n), n), n)

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        tema (string): the column name for the n-day double exponential moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with n-day double exponential moving average of the asset calculated.

    """

    df = ema(df, price, tema + "_ema", n)
    df = ema(df[n - 1 :], tema + "_ema", tema + "_ema_2", n)
    df = ema(df[n - 1 :], tema + "_ema_2", tema + "_ema_3", n)
    df[tema] = (
        3 * df[tema + "_ema"] - 3 * df[tema + "_ema_2"] + df[tema + "_ema_3"]
    )
    df = df.dropna().reset_index(drop=True)
    df.drop(
        [tema + "_ema", tema + "_ema_2", tema + "_ema_3"], axis=1, inplace=True
    )

    return df



import pandas as pd


def midpnt(df, price, midpnt, n):
    """
    The Midpoint calculation is similar to the Midprice, except the highest and
    lowest values are returned from the same input field. The default indicator
    calculates the highest close and lowest close within the look back period
    and averages the two values.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the price of the asset.
        midpnt (string): the column name for the calculated midpoint values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with the midpoint calculated.

    """

    midpnt_max = df[price].rolling(window=n).max()
    midpnt_min = df[price].rolling(window=n).min()
    df[midpnt] = (midpnt_max + midpnt_min) / 2
    df = df.dropna().reset_index(drop=True)

    return df



from indicators.momentum.ao import ao
import pandas as pd
from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def accelerator_oscillator(df,high, low, col, ma_type, n =5, col='ac'):
   
    ma_types = {"sma":sma, "ema" : ema, "wma": wma, "dema": dema, "tema": tema}

    # Data frame for storing temporary data
    df_tmp = pd.DataFrame()
    df_tmp[high] = df[high]
    df_tmp[low] = =df[low]

        # Calculate Awesome Oscillator
    df_tmp = ao(df_tmp, high, low, col, ma_type, 'ao')

        # Calculate SMA for Awesome Oscillator
    df_tmp = calculate_sma(df_tmp, 'ao' , 'sma_ao', n)

        # Calculate Accelerator Oscillator
    df_tmp[column_name] = df_tmp['ao'] - df_tmp['sma_ao']
    df_tmp = df_tmp[[column_name]]
    df = df.merge(df_tmp, left_index=True, right_index=True)

    return df



import pandas as pd


def roc(df, price, col, n):
    """
    The Price Rate of Change (ROC) is a momentum-based technical indicator that measures the percentage change in price between the current price 
    and the price a certain number of periods ago. The ROC indicator is plotted against zero, 
    with the indicator moving upwards into positive territory if price changes are to the upside, 
    and moving into negative territory if price changes are to the downside.

    
    ROC=((close - close_n )/close_n ) * 100

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name of the price of the asset.
        col (string): the column name for the rate of change values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with roc of the asset calculated.

    Strategy: ROC smaller than 0 is typically a sell signal and ROC above 0 is typically a buy signal
    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df

    price_shift_n = df[price].shift(n)
    df[col] = ((df[price] - price_shift_n) / price_shift_n) * 100
    #df = df.dropna().reset_index(drop=True)

    return df




import pandas as pd


def midpri(df, high, low, midpri, n):
    """
    The Midprice returns the midpoint value from two different input fields. The
    default indicator calculates the highest high and lowest low within the look
    back period and averages the two values to return the Midprice.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the price of the asset.
        midpri (string): the column name for the calculated midprice values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with the midprice calculated.

    """

    midpri_hh = df[high].rolling(window=n).max()
    midpri_ll = df[low].rolling(window=n).min()
    df[midpri] = (midpri_hh + midpri_ll) / 2
    df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd
import numpy as np

def ichimoku_a(df, high, low, col, n1=9, n2=26):
    """Ichimoku Kinkō Hyō (Ichimoku)
    It identifies the trend and look for potential signals within that trend.
    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
    Args:
        df: dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        col: Column name for result column.
        n1: fast n period
        n2:medium  n period.
        
    Returns:
        pandas.Series: New column generated.
    """

    df = df.copy()
    conv = 0.5 * (df[high].rolling(n1).max() + df[low].rolling(n1).min())
    base = 0.5 * (df[high].rolling(n2).max() + df[low].rolling(n2).min())

    spana = 0.5 * (conv + base)
    spana = spana.shift(n2)
    df[col] = spana
    return df


def ichimoku_b(df, high, low, col, n2=26, n3=52):
    """Ichimoku Kinkō Hyō (Ichimoku)
    It identifies the trend and look for potential signals within that trend.
    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
    Args:
        df: dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        col: Column name for result column.
        n2:medium  n period.
        n3: slow n period
    Returns:
        pandas.Series: New column generated.
    """

    df = df.copy()

    spanb = 0.5 * (df[high].rolling(n3).max() + df[low].rolling(n3).min())
    spanb = spanb.shift(n2)
    df[col] = spanb
    return df

def ichimoku_base(df,high,low,col,n2=26):
    """
    Kijun-sen (Base Line): (26-period high + 26-period low)/2))

    The default setting is 26 periods and can be adjusted. On a daily chart, this line is the midpoint of the 26-day high-low range, 
    which is almost one month).  

    Args:
        df: dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        col: Column name for result column.
        n2(int): n period.
    Returns:
        pandas.Series: New column generated.

    """

    df = df.copy()
    high_prices = df[high].rolling(n2).max()
    low_prices = df[low].rolling(n2).min()
    df[col] = (high_prices + low_prices) / 2
    return df






import pandas as pd


def ad(df, high, low, close, volume, col):
    """
    Accumulation/distribution is a cumulative indicator that uses volume and price to assess whether a stock is being accumulated or distributed. 
    The accumulation/distribution measure seeks to identify divergences between the stock price and volume flow. 
    This provides insight into how strong a trend is. If the price is rising but the indicator is falling this 
    indicates that buying or accumulation volume may not be enough to support the price rise and a price decline could be forthcoming.
    The CLV is based on the movement of the issue within a single bar and can be +1, -1 or zero.
    
    A/D=Previous A/D+Current Money Flow Value
    CMFV = ((Close - Low) - (high - close)//(high - Low)) * Volume

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        volume (string): the column name for the volume of the asset.
        col (string): the column name for the ad values.

    Returns:
        df (pd.DataFrame): Dataframe with ad of the asset calculated.

    Sample Strategy:
   
    Bullish:

	AND [Daily SMA(60,Daily Volume) > 100000] 
	AND [Daily SMA(60,Daily Close) > 10] 

	AND [Daily Close < Daily SMA(65,Daily Close)] 
	AND [Daily AccDist > Daily AccDist Signal (65)] 
	AND [Daily OBV > Daily OBV Signal(65)] 
	AND [Daily Close < Daily SMA(20,Daily Close)] 
	AND [Daily AccDist > Daily AccDist Signal (20)] 
	AND [Daily OBV > Daily OBV Signal(20)]

	Bearish:
	AND [Daily SMA(60,Daily Volume) > 100000] 
	AND [Daily SMA(60,Daily Close) > 10] 

	AND [Daily Close > Daily SMA(65,Daily Close)] 
	AND [Daily AccDist < Daily AccDist Signal (65)] 
	AND [Daily OBV < Daily OBV Signal(65)] 
	AND [Daily Close > Daily SMA(20,Daily Close)] 
	AND [Daily AccDist < Daily AccDist Signal (20)] 
	AND [Daily OBV < Daily OBV Signal(20)]
	
	Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:accumulation_distribution_line
    """

    money_flow_multiplier = (
        (df[close] - df[low]) - (df[high] - df[close])
    ) / (df[high] - df[low])
    df[col + "_money_flow_volume"] = money_flow_multiplier * df[volume]
    prev_ad = df.loc[0, col + "_money_flow_volume"]
    df.loc[0, col] = prev_ad
    ads = [0.0]
    for row in df.loc[1:, [col + "_money_flow_volume"]].itertuples(index=False):
        ads.append(prev_ad + row[0])
        prev_ad = ads[-1]
    df = df.fillna(0)
    df[col] += ads
    df.drop([col + "_money_flow_volume"], axis=1, inplace=True)

    return df



import pandas as pd


def mfi(df, high, low, close, volume, col, n):
    """
    The Money Flow Index (MFI) is an oscillator that uses both price and volume to measure buying and selling pressure.
    Money flow is positive when the typical price rises (buying pressure) and negative when the typical price declines (selling pressure). 
    A ratio of positive and negative money flow is then plugged into an RSI formula to create an oscillator that moves between zero and one hundred. 
    As a momentum oscillator tied to volume, MFI is best suited to identify reversals and price extremes with a variety of signals.
    
    Typical Price = (High + Low + Close)/3

    Raw Money Flow = Typical Price x Volume
    Money Flow Ratio = (14-period Positive Money Flow)/(14-period Negative Money Flow)

    Money Flow Index = 100 - 100/(1 + Money Flow Ratio)


    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        volume (string): the column name of the volume of the asset.
        mfi (string): the column name for the mfi values.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with mfi of the asset calculated.

    Strategy:
     - Bullish
        AND [Daily SMA(20,Daily Volume) > 40000] 
        AND [Daily SMA(60,Daily Close) > 20] 
        AND [Daily MFI(14) < 10]

     - Bearish

        AND [Daily SMA(20,Daily Volume) > 100000] 
        AND [Daily SMA(60,Daily Close) > 20] 

        AND [Daily MFI(14) > 90]

    Citation: https://school.stockcharts.com/doku.php?id=technical_indicators:money_flow_index_mfi


    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df


    typical_price = (df[high] + df[low] + df[close]) / 3
    money_flow = typical_price * df[volume]
    typical_price_diff = typical_price.diff()
    df.loc[typical_price_diff > 0, "positive_money_flow"] = 1
    df.loc[typical_price_diff <= 0, "negative_money_flow"] = -1
    df["positive_money_flow"] *= money_flow
    df["negative_money_flow"] *= money_flow
    df = df.fillna(0)
    n_pos_money_flow = (
        df.loc[1:, "positive_money_flow"].rolling(window=n).sum()
    )
    n_neg_money_flow = (
        df.loc[1:, "negative_money_flow"].rolling(window=n).sum()
    )
    mfi_ratio = n_pos_money_flow / -n_neg_money_flow
    df[col] = 100 - (100 / (1 + mfi_ratio))
    df.drop(
        ["positive_money_flow", "negative_money_flow"], axis=1, inplace=True
    )
    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd
import numpy as np


def vortex_pos(df, high, low, close, col, n=14):
    """Vortex Indicator (VI)
    It consists of two oscillators that capture positive and negative trend
    movement. A bullish signal triggers when the positive trend indicator
    crosses above the negative trend indicator or a key level.
    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:vortex_indicator
    Args:
        high(pandas.Series): dataset 'High' column.
        low(pandas.Series): dataset 'Low' column.
        close(pandas.Series): dataset 'Close' column.
        n(int): n period.
        fillna(bool): if True, fill nan values.
    Returns:
        pandas.Series: New feature generated.
    """
    tr = df[high].combine(df[close].shift(1), max) - df[low].combine(df[close].shift(1), min)
    trn = tr.rolling(n).sum()

    vmp = np.abs(df[high] - df[low].shift(1))
    vmm = np.abs(df[low] - df[high].shift(1))

    vip = vmp.rolling(n).sum() / trn

    df[col] = vip
    return df


def vortex_neg(df, high, low, close, col, n=14):

    """Vortex Indicator (VI)
    It consists of two oscillators that capture positive and negative trend
    movement. A bearish signal triggers when the negative trend indicator
    crosses above the positive trend indicator or a key level.
    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:vortex_indicator
    Args:
        high(pandas.Series): dataset 'High' column.
        low(pandas.Series): dataset 'Low' column.
        close(pandas.Series): dataset 'Close' column.
        n(int): n period.
        fillna(bool): if True, fill nan values.
    Returns:
        pandas.Series: New feature generated.
    """

    df = df.copy()
    tr = df[high].combine(df[close].shift(1), max) - df[low].combine(df[close].shift(1), min)
    trn = tr.rolling(n).sum()

    vmp = np.abs(df[high] - df[low].shift(1))
    vmm = np.abs(df[low] - df[high].shift(1))

    vin = vmm.rolling(n).sum() / trn

    df[col] = vin
    return df


import pandas as pd
import numpy as np

import sys

from voyant_indicators.volatility.atr import atr


def SuperTrend(df, period, multiplier, ohlc=['Open', 'High', 'Low', 'Close']):
    """
    Function to compute SuperTrend
    
    Args :
        df : Pandas DataFrame which contains ['date', 'open', 'high', 'low', 'close', 'volume'] columns
        period : Integer indicates the period of computation in terms of number of candles
        multiplier : Integer indicates value to multiply the ATR
        ohlc: List defining OHLC Column names (default ['Open', 'High', 'Low', 'Close'])
        
    Returns :
        df : Pandas DataFrame with new columns added for 
            True Range (TR), ATR (ATR_$period)
            SuperTrend (ST_$period_$multiplier)
            SuperTrend Direction (STX_$period_$multiplier)
    """

    df = df.copy()

    df = atr(df, "High", "Low", "Close", "TR", period)
    atr_name = 'TR'
    st = 'ST_' + str(period) + '_' + str(multiplier)
    stx = 'STX_' + str(period) + '_' + str(multiplier)
    
    """
    SuperTrend Algorithm :
    
        BASIC UPPERBAND = (HIGH + LOW) / 2 + Multiplier * ATR
        BASIC LOWERBAND = (HIGH + LOW) / 2 - Multiplier * ATR
        
        FINAL UPPERBAND = IF( (Current BASICUPPERBAND < Previous FINAL UPPERBAND) or (Previous Close > Previous FINAL UPPERBAND))
                            THEN (Current BASIC UPPERBAND) ELSE Previous FINALUPPERBAND)
        FINAL LOWERBAND = IF( (Current BASIC LOWERBAND > Previous FINAL LOWERBAND) or (Previous Close < Previous FINAL LOWERBAND)) 
                            THEN (Current BASIC LOWERBAND) ELSE Previous FINAL LOWERBAND)
        
        SUPERTREND = IF((Previous SUPERTREND = Previous FINAL UPPERBAND) and (Current Close <= Current FINAL UPPERBAND)) THEN
                        Current FINAL UPPERBAND
                    ELSE
                        IF((Previous SUPERTREND = Previous FINAL UPPERBAND) and (Current Close > Current FINAL UPPERBAND)) THEN
                            Current FINAL LOWERBAND
                        ELSE
                            IF((Previous SUPERTREND = Previous FINAL LOWERBAND) and (Current Close >= Current FINAL LOWERBAND)) THEN
                                Current FINAL LOWERBAND
                            ELSE
                                IF((Previous SUPERTREND = Previous FINAL LOWERBAND) and (Current Close < Current FINAL LOWERBAND)) THEN
                                    Current FINAL UPPERBAND
    """
    
    # Compute basic upper and lower bands
    df['basic_ub'] = (df[ohlc[1]] + df[ohlc[2]]) / 2 + multiplier * df[atr_name]
    df['basic_lb'] = (df[ohlc[1]] + df[ohlc[2]]) / 2 - multiplier * df[atr_name]

    # Compute final upper and lower bands
    df['final_ub'] = 0.00
    df['final_lb'] = 0.00
    for i in range(period, len(df)):
        df['final_ub'].iat[i] = df['basic_ub'].iat[i] if df['basic_ub'].iat[i] < df['final_ub'].iat[i - 1] or df[ohlc[3]].iat[i - 1] > df['final_ub'].iat[i - 1] else df['final_ub'].iat[i - 1]
        df['final_lb'].iat[i] = df['basic_lb'].iat[i] if df['basic_lb'].iat[i] > df['final_lb'].iat[i - 1] or df[ohlc[3]].iat[i - 1] < df['final_lb'].iat[i - 1] else df['final_lb'].iat[i - 1]
       
    # Set the Supertrend value
    df[st] = 0.00
    for i in range(period, len(df)):
        df[st].iat[i] = df['final_ub'].iat[i] if df[st].iat[i - 1] == df['final_ub'].iat[i - 1] and df[ohlc[3]].iat[i] <= df['final_ub'].iat[i] else \
                        df['final_lb'].iat[i] if df[st].iat[i - 1] == df['final_ub'].iat[i - 1] and df[ohlc[3]].iat[i] >  df['final_ub'].iat[i] else \
                        df['final_lb'].iat[i] if df[st].iat[i - 1] == df['final_lb'].iat[i - 1] and df[ohlc[3]].iat[i] >= df['final_lb'].iat[i] else \
                        df['final_ub'].iat[i] if df[st].iat[i - 1] == df['final_lb'].iat[i - 1] and df[ohlc[3]].iat[i] <  df['final_lb'].iat[i] else 0.00 
                 
    # Mark the trend direction up/down
    df[stx] = np.where((df[st] > 0.00), np.where((df[ohlc[3]] < df[st]), 'down',  'up'), np.NaN)

    # Remove basic and final bands from the columns
    df.drop(['basic_ub', 'basic_lb', 'final_ub', 'final_lb'], inplace=True, axis=1)
    
    df = df.fillna(0, inplace=False)

    return df



import pandas as pd


def trange(df, high, low, close, tr):
    """
    Welles Wilder described these calculations to determine the trading range
    for a stock or commodity. True Range is defined as the largest of the
    following:

    - The distance from today's high to today's low.
    - The distance from yesterday's close to today's high.
    - The distance from yesterday's close to today's low.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        tr (string): the column name for the tr values.

    Returns:
        df (pd.DataFrame): Dataframe with the true range calculated.

    """
    
    prev_close = df[close].shift(1)
    df["h-l"] = df[high] - df[low]
    df["h-pc"] = df[high] - prev_close
    df["pc-l"] = prev_close - df[low]
    df = df.dropna().reset_index(drop=True)
    df[tr] = df[["h-l", "h-pc", "pc-l"]].max(axis=1)
    df.drop(["h-l", "h-pc", "pc-l"], axis=1, inplace=True)

    return df



import pandas as pd

def fi(df,close, volume, col, n=2):
    """Force Index (FI)
    It illustrates how strong the actual buying or selling pressure is. High
    positive values mean there is a strong rising trend, and low values signify
    a strong downward trend.
    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:force_index
    Args:
        close(pandas.Series): dataset 'Close' column.
        volume(pandas.Series): dataset 'Volume' column.
        n(int): n period.
        fillna(bool): if True, fill nan values.
    Returns:
        pandas.Series: New feature generated.
    """
    df = df.copy()

    fi = df[close].diff(n) * df[volume].diff(n)
    
    df[col] = fi
    return df



import pandas as pd
from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma


def dema(df, price, col, n):
    """
    Double Exponential Moving Average (DEMA) attempts to offer a smoothed average
    with less lag than a straight exponential moving average.

    The DEMA equation doubles the EMA, but then cancels out the lag by subtracting
    the square of the EMA.

    DEMA = 2 * EMA(p, n) - EMA(EMA(p, n), n)

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        dema (string): the column name for the n-day double exponential moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with n-day double exponential moving average of the asset calculated.


    """


    df = df.copy().reset_index(drop=True)

    if df.shape[0] < n:
        df[col] = None
        return df

    df = ema(df, price, col + "_ema", n)
    df = ema(df[n - 1 :], col + "_ema", col + "_ema_2", n)
    df[col] = 2 * df[col + "_ema"] - df[col + "_ema_2"]

    print(df)
    #df = df.dropna().reset_index(drop=True)
    df.drop([col + "_ema", col + "_ema_2"], axis=1, inplace=True)

    return df





import pandas as pd
from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def bbands(df, price, bbands, n_bdev_up, n_bdev_dn, ma_type, n):
    """
    Bollinger Bands are volatility bands placed above and below a moving average.
    Volatility is based on the standard deviation, which changes as volatility
    increases and decreases. The bands automatically widen when volatility
    increases and narrow when volatility decreases.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the desired price type of the asset.
        bbands (string): the column name for the bbands values.
        n_bdev_up (int): the standard deviation multiplier of the upper band.
        n_bdev_dn (int): the standard deviation multiplier of the lower band.
        ma_type (int): moving average type of the time series.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with average true range of the asset calculated.

    """

    ma_types = {0: sma, 1: ema, 2: wma, 3: dema, 4: tema}

    n_days_std = df[price].rolling(window=n).std(ddof=0)
    df = ma_types[ma_type](df, price, bbands + "_middle_band", n)
    df[bbands + "_upper_band"] = df[bbands + "_middle_band"] + (
        n_days_std * n_bdev_up
    )
    df[bbands + "_lower_band"] = df[bbands + "_middle_band"] - (
        n_days_std * n_bdev_dn
    )
    df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema

def stochf(df, high, low, close, fast_k_n, fast_d_n, fast_d_ma_type=0,col,indicator_val):
    """
    The Stochastic Oscillator is another well-known momentum indicator used in
    technical analysis. The idea behind this indicator is that the closing
    prices should predominantly close in the same direction as the prevailing
    trend.

    In an upward trend the price should be closing near the highs of the trading
    range and in a downward trend the price should be closing near the lows of
    the trading range. When this occurs it signals continued momentum and
    strength in the direction of the prevailing trend.

    Fast Stochastic Oscillator:
        Fast %K = %K basic calculation
        Fast %D = n period moving average of Fast %K

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        high (string): the column name for the period highest price  of the asset.
        low (string): the column name for the period lowest price of the asset.
        close (string): the column name for the closing price of the asset.
        fast_k_n (int): the time period of the fast k moving average.
        fast_d_n (int): the time period of the fast d moving average.
        fast_d_ma_type (int): Moving average type for the slow d moving average.

    Returns:
        df (pd.DataFrame): Dataframe with fast %k and fast %d of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < n:
        df[col] = None
        return df


    ma_types = {0: sma, 1: ema, 2: wma, 3: dema, 4: tema}

    hh = df[high].rolling(window=fast_k_n).max()
    ll = df[low].rolling(window=fast_k_n).min()
    df["fast_%k"] = (df[close] - ll) / (hh - ll) * 100
    df = ma_types[fast_d_ma_type](
        df[fast_k_n - 1 :], "fast_%k", "fast_%d", fast_d_n
    )

    df[col] = 0

    #df = df.dropna().reset_index(drop=True)

    return df



import pandas as pd
import numpy as np

def adx(df, high, low, close, col, n=14):
    """Calculate the Average Directional Movement Index for given data.


    The Average Directional Index (ADX), Minus Directional Indicator (-DI) and Plus Directional Indicator (+DI) represent a group of directional movement indicators that form a trading system developed by Welles Wilder. 
    Positive and negative directional movement form the backbone of the Directional Movement System. 
    Wilder determined directional movement by comparing the difference between two consecutive lows with the difference between their respective highs.
    The Plus Directional Indicator (+DI) and Minus Directional Indicator (-DI) are derived from smoothed averages of these differences and 
    measure trend direction over time. These two indicators are often collectively referred to as the Directional Movement Indicator (DMI).
    The Average Directional Index (ADX) is in turn derived from the smoothed averages of the difference between +DI and -DI; 
    it measures the strength of the trend (regardless of direction) over time.
    Using these three indicators together, chartists can determine both the direction and strength of the trend.


    
    Args:
        df= dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        close: Column name for 'Close' column.
        n(int): n period.
    Returns:
        pandas.Series: New feature generated.

    Strategy:
    Buy Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Plus DI(14) crosses Daily Minus DI(14)] 
    AND [Daily Close > Daily SMA(50,Daily Close)]
    
    SELL Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Minus DI(14) crosses Daily Plus DI(14)] 
    AND [Daily Close < Daily SMA(50,Daily Close)]


    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:average_directional_index_adx
    
    """
    n = int(n)

    df = df.copy()

    cs = df[close].shift(1)

    tr = df[high].combine(cs, max) - df[low].combine(cs, min)
    trs = tr.rolling(n).sum()

    up = df[high] - df[high].shift(1)
    dn = df[low].shift(1) - df[low]

    pos = ((up > dn) & (up > 0)) * up
    neg = ((dn > up) & (dn > 0)) * dn

    dip = 100 * pos.rolling(n).sum() / trs
    din = 100 * neg.rolling(n).sum() / trs

    dx = 100 * np.abs((dip - din) / (dip + din))
    adx = dx.ewm(n).mean()

    df[col] = adx
    return df



def adx_pos(df, high, low, close,col,n=14):
    """Calculate the Average Directional Movement Index for given data.


    The Average Directional Index (ADX), Minus Directional Indicator (-DI) and Plus Directional Indicator (+DI) represent a group of directional movement indicators that form a trading system developed by Welles Wilder. 
    Positive and negative directional movement form the backbone of the Directional Movement System. 
    Wilder determined directional movement by comparing the difference between two consecutive lows with the difference between their respective highs.
    The Plus Directional Indicator (+DI) and Minus Directional Indicator (-DI) are derived from smoothed averages of these differences and 
    measure trend direction over time. These two indicators are often collectively referred to as the Directional Movement Indicator (DMI).
    The Average Directional Index (ADX) is in turn derived from the smoothed averages of the difference between +DI and -DI; 
    it measures the strength of the trend (regardless of direction) over time.
    Using these three indicators together, chartists can determine both the direction and strength of the trend.


    
    Args:
        df= dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        close: Column name for 'Close' column.
        n(int): n period.
    Returns:
        pandas.Series: New feature generated.

    Strategy:
    Buy Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Plus DI(14) crosses Daily Minus DI(14)] 
    AND [Daily Close > Daily SMA(50,Daily Close)]
    
    SELL Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Minus DI(14) crosses Daily Plus DI(14)] 
    AND [Daily Close < Daily SMA(50,Daily Close)]


    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:average_directional_index_adx
    
    """

    n = int(n)

    df = df.copy()

    cs = df[close].shift(1)

    tr = df[high].combine(cs, max) - df[low].combine(cs, min)
    trs = tr.rolling(n).sum()

    up = df[high] - df[high].shift(1)
    dn = df[low].shift(1) - df[low]

    pos = ((up > dn) & (up > 0)) * up
    neg = ((dn > up) & (dn > 0)) * dn

    dip = 100 * pos.rolling(n).sum() / trs
    din = 100 * neg.rolling(n).sum() / trs

    dx = 100 * np.abs((dip - din) / (dip + din))
    adx = dx.ewm(n).mean()

    df['adx_pos'] = dip

    return df



def adx_neg(df, high, low, close, col, n=14):
    """Calculate the Average Directional Movement Index for given data.


    The Average Directional Index (ADX), Minus Directional Indicator (-DI) and Plus Directional Indicator (+DI) represent a group of directional movement indicators that form a trading system developed by Welles Wilder. 
    Positive and negative directional movement form the backbone of the Directional Movement System. 
    Wilder determined directional movement by comparing the difference between two consecutive lows with the difference between their respective highs.
    The Plus Directional Indicator (+DI) and Minus Directional Indicator (-DI) are derived from smoothed averages of these differences and 
    measure trend direction over time. These two indicators are often collectively referred to as the Directional Movement Indicator (DMI).
    The Average Directional Index (ADX) is in turn derived from the smoothed averages of the difference between +DI and -DI; 
    it measures the strength of the trend (regardless of direction) over time.
    Using these three indicators together, chartists can determine both the direction and strength of the trend.


    
    Args:
        df= dataframe
        high:  Column name for 'High' column.
        low: Column name for 'Low' column.
        close: Column name for 'Close' column.
        n(int): n period.
    Returns:
        pandas.Series: New feature generated.

    Strategy:
    Buy Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Plus DI(14) crosses Daily Minus DI(14)] 
    AND [Daily Close > Daily SMA(50,Daily Close)]
    
    SELL Signal:

    AND [Daily SMA(20,Daily Volume) > 100000] 
    AND [Daily SMA(60,Daily Close) > 10] 

    AND [Daily ADX Line(14) > 20] 
    AND [Daily Minus DI(14) crosses Daily Plus DI(14)] 
    AND [Daily Close < Daily SMA(50,Daily Close)]


    http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:average_directional_index_adx
    
    """

    n = int(n)

    df = df.copy()

    cs = df[close].shift(1)

    tr = df[high].combine(cs, max) - df[low].combine(cs, min)
    trs = tr.rolling(n).sum()

    up = df[high] - df[high].shift(1)
    dn = df[low].shift(1) - df[low]

    pos = ((up > dn) & (up > 0)) * up
    neg = ((dn > up) & (dn > 0)) * dn

    dip = 100 * pos.rolling(n).sum() / trs
    din = 100 * neg.rolling(n).sum() / trs

    dx = 100 * np.abs((dip - din) / (dip + din))
    adx = dx.ewm(n).mean()

    df['adx_neg'] = din

    return df



import pandas as pd


def kama(df, price, kama, n, fast_ema=2, slow_ema=30):
    """
    Kaufman's Adaptive Moving Average (KAMA) is a moving average designed to
    account for market noise or volatility. KAMA will closely follow prices when
    the price swings are relatively small and the noise is low. KAMA will adjust
    when the price swings widen and follow prices from a greater distance.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name for the series type of the asset.
        kama (string): the column name for the kama results.
        n (int): the total number of periods.
        fast_ema (int): the time period of the fast exponential moving average.
        slow_ema (int): the time period of the slow exponential moving average.

    Returns:
        df (pd.DataFrame): Dataframe with kama of the asset calculated.

    """

    er = (
        df[price].diff(n).abs()
        / df[price].diff().abs().rolling(window=n).sum()
    )
    fast_sc = 2 / (fast_ema + 1)
    slow_sc = 2 / (slow_ema + 1)
    df[kama + "_sc"] = ((er * (fast_sc - slow_sc)) + slow_sc) ** 2
    prev_kama = list(df[:n][price].rolling(window=n).mean())[-1]
    df.loc[n - 1, kama] = prev_kama
    df.loc[n:, kama] = 0.0
    kamas = [0.0 for i in range(n)]
    for row in df.loc[n:, [price, kama + "_sc"]].itertuples(index=False):
        kamas.append(prev_kama + row[1] * (row[0] - prev_kama))
        prev_kama = kamas[-1]
    df[kama] += kamas
    df = df.dropna().reset_index(drop=True)
    df.drop([kama + "_sc"], axis=1, inplace=True)

    return df




import pandas as pd

from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema

from indicators.momentum.roc import roc


def trix(df, price, trix, n):
    """
    TRIX is a momentum oscillator that displays the percent rate of change of a
    triple exponentially smoothed moving average.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset price.
        price (string): the column name of the price of the asset.
        trix (string): the column name for the rate of change of a triple exponential moving average results.
        n (int): the total number of periods.

    Returns:
        df (pd.DataFrame): Dataframe with the rate of change of a triple exponential moving average of the asset calculated.

    """

    df = ema(df, price, trix + "_ema", n)
    df = ema(df[n - 1 :], trix + "_ema", trix + "_ema_2", n)
    df = ema(df[n - 1 :], trix + "_ema_2", trix + "_ema_3", n)
    df = roc(df, trix + "_ema_3", trix, 1)

    return df



import pandas as pd
from voyant_indicators.overlapping_studies.sma import sma
from voyant_indicators.overlapping_studies.ema import ema
from voyant_indicators.overlapping_studies.wma import wma
from voyant_indicators.overlapping_studies.dema import dema
from voyant_indicators.overlapping_studies.tema import tema


def apo(df, price, col, fast_period, slow_period, ma_type):
    """
    The Absolute Price Oscillator (APO) shows the difference between two moving
    averages. It is basically a MACD, but the Price Oscillator can use any time
    periods. A buy signal is generated when the Price Oscillator rises above
    zero, and a sell signal when the it falls below zero.

    Parameters:
        df (pd.DataFrame): DataFrame which contain the asset information.
        price (string): the column name of the price of the asset.
        fast_period (int): the time period of the fast moving average.
        slow_period (int): the time period of the slow moving average.
        ma_type (int): Moving average type.

    Returns:
        df (pd.DataFrame): Dataframe with apo of the asset calculated.

    """

    df = df.copy().reset_index(drop=True)
    if df.shape[0] < slow_period:
        df[col] = None
        return df

    ma_types = {"sma":sma, "ema" : ema, "wma": wma, "dema": dema, "tem": tema}

    df = ma_types[ma_type](df, price, col + "_fast", fast_period)
    df = ma_types[ma_type](df, price, col + "_slow", slow_period)
    df[col] = df[col + "_fast"] - df[col + "_slow"]
    df = df.dropna().reset_index(drop=True)

    return df



def de_marker(df,high, low col, period=14):
        """
        DeMarker (DeM)
        --------------
            https://www.metatrader4.com/en/trading-platform/help/analytics/tech_indicators/demarker
            >>> Indicators.de_marker(period=14, column_name='dem')
            :param int period: Period, default: 14
            :param str column_name: Column name, default: dem
            :return: None
        """
    df_tmp = df[[high, low]]

    df_tmp = df_tmp.assign(
        hdif=(df_tmp[high] > df_tmp[high].shift(1)).astype(int))
    df_tmp = df_tmp.assign(hsub=df_tmp[high] - df_tmp[high].shift(1))
    df_tmp = df_tmp.assign(demax=np.where(df_tmp.hdif == 0, 0, df_tmp.hsub))

    df_tmp = df_tmp.assign(ldif=(df_tmp[low] < df_tmp[low].shift(1)).astype(int))
    df_tmp = df_tmp.assign(lsub=df_tmp[low].shift(1) - df_tmp[low])
    df_tmp = df_tmp.assign(demin=np.where(df_tmp.ldif == 0, 0, df_tmp.lsub))

    df_tmp['sma_demax'] = df_tmp['demax'].rolling(window=period).mean()
    df_tmp['sma_demin'] = df_tmp['demin'].rolling(window=period).mean()

    df_tmp = df_tmp.assign(dem=df_tmp.sma_demax / (df_tmp.sma_demax + df_tmp.sma_demin))

    df_tmp = df_tmp[['dem']]
    df_tmp = df_tmp.rename(columns={'dem': col})

    df = df.merge(df_tmp, left_index=True, right_index=True)

    return df